@@ -18,6 +18,7 @@
  * DAMAGE.
  */
 
+#include <dirent.h>
 #include <dlfcn.h>
 #include <errno.h>
 #include <stdlib.h>
@@ -28,14 +29,17 @@
 #include <sys/mman.h>
 #include <unistd.h>
 
-#include "logger.h"
 #include "acvpproxy.h"
+#include "fips.h"
 #include "internal.h"
 #include "json_wrapper.h"
 #include "definition.h"
+#include "logger.h"
 #include "memset_secure.h"
+#include "request_helper.h"
 #include "threading_support.h"
 #include "totp.h"
+#include "totp_mq_server.h"
 
 /*****************************************************************************
  * Globals
@@ -542,6 +546,7 @@ void acvp_ctx_release(struct acvp_ctx *ctx)
 	acvp_release_modinfo(&ctx->modinfo);
 	acvp_release_datastore(&ctx->datastore);
 	acvp_release_search(search);
+	acvp_release_auth_ctx(ctx);
 
 	free(ctx);
 }
@@ -627,6 +632,8 @@ int acvp_ctx_init(struct acvp_ctx **ctx,
 	CKINT(acvp_duplicate(&datastore->srcserver, ACVP_DS_SRCSERVER));
 	CKINT(acvp_duplicate(&datastore->expectedfile, ACVP_DS_EXPECTED));
 
+	CKINT(acvp_init_auth_ctx(*ctx));
+
 out:
 	return ret;
 }
@@ -650,13 +657,10 @@ void acvp_release(void)
 
 DSO_PUBLIC
 int acvp_init(const uint8_t *seed, uint32_t seed_len, time_t last_gen,
-	      void (*last_gen_cb)(time_t now))
+	      bool production, void (*last_gen_cb)(time_t now))
 {
 	int ret;
 
-	CKNULL_LOG(seed, -EINVAL, "TOTP seed missing\n");
-	CKNULL_LOG(seed_len, -EINVAL, "TOTP seed length is zero\n");
-
 	if (!ds) {
 		logger(LOGGER_ERR, LOGGER_C_ANY,
 		       "No datastore backend registered\n");
@@ -676,7 +680,10 @@ int acvp_init(const uint8_t *seed, uint32_t seed_len, time_t last_gen,
 	if (!acvp_library_initialized())
 		CKINT(thread_init(2));
 
-	CKINT(totp_set_seed(seed, seed_len, last_gen, last_gen_cb));
+	if (seed && seed_len)
+		CKINT(totp_set_seed(seed, seed_len, last_gen, production,
+				    last_gen_cb));
+
 	CKINT(sig_install_handler());
 
 	acvp_op_enable();
@@ -694,10 +701,13 @@ int acvp_load_extension(const char *path)
 	int ret = 0;
 	void *library_handle = NULL;
 
+	CKNULL_LOG(path, -EINVAL, "Pathname missing\n");
+
+	CKINT(fips_post_integrity(path));
+
 	library_handle = dlopen(path, RTLD_NOW);
 	CKNULL_LOG(library_handle, -EFAULT, "Error loading library: %s\n",
 		   dlerror());
-
 	dlerror();
 	extension = (struct acvp_extension *)dlsym(library_handle,
 						   "acvp_extension");
@@ -708,3 +718,141 @@ int acvp_load_extension(const char *path)
 out:
 	return ret;
 }
+
+/* Return true if extension version is greater or equal to provided number. */
+static bool acvp_so_version_ge(const struct dirent *dentry,
+			       unsigned int maj, unsigned int minor,
+			       unsigned int patchlevel)
+{
+	size_t filenamelen, i, end = 0;
+	unsigned long f_maj = 0, f_minor = 0, f_patchlevel = 0;
+	bool maj_done = false, minor_done = false, patchlevel_done = false;
+	char tmp[4];
+
+	/* Check that entry is neither ".", "..", or a hidden file */
+	if (dentry->d_name[0] == '.')
+		return false;
+
+	/* Check that it is a regular file or a symlink */
+	if (dentry->d_type != DT_REG && dentry->d_type != DT_LNK)
+		return false;
+
+	filenamelen = strlen(dentry->d_name);
+	if (!filenamelen)
+		return false;
+
+	memset(tmp, 0, sizeof(tmp));
+	for (i = filenamelen - 1; i > 0; i--) {
+		size_t diff;
+
+		/* Version numbers are bullet-separated. */
+		if (dentry->d_name[i] != '.')
+			continue;
+
+		/* Skip the extension of the file name. */
+		if (!end) {
+			end = i;
+			continue;
+		}
+
+		diff = end - i - 1;
+
+		/* Ensure that there are no two dots next to each other. */
+		if (diff == 0) {
+			end = i;
+			continue;
+		}
+
+		/* Number shall have at most 4 digits. */
+		if (diff > sizeof(tmp))
+			diff = sizeof(tmp);
+
+		/* Copy the version into tmp variable. */
+		memcpy(tmp, dentry->d_name + i + 1, diff);
+
+		end = i;
+
+		/* Convert data into integer. */
+		if (!patchlevel_done) {
+			f_patchlevel = strtoul(tmp, NULL, 10);
+			patchlevel_done = true;
+			memset(tmp, 0, sizeof(tmp));
+			if (f_patchlevel == ULONG_MAX)
+				return false;
+			continue;
+		} else if (!minor_done) {
+			f_minor = strtoul(tmp, NULL, 10);
+			minor_done = true;
+			memset(tmp, 0, sizeof(tmp));
+			if (f_minor == ULONG_MAX)
+				return false;
+			continue;
+		} else if (!maj_done) {
+			f_maj = strtoul(tmp, NULL, 10);
+			maj_done = true;
+			memset(tmp, 0, sizeof(tmp));
+			if (f_maj == ULONG_MAX)
+				return false;
+		}
+
+		/* Perform matching of version information. */
+		if (maj < f_maj)
+			return true;
+		if (maj == f_maj) {
+			if (minor < f_minor)
+				return true;
+			if (minor == f_minor) {
+				if (patchlevel <= f_patchlevel)
+					return true;
+			}
+		}
+		return false;
+	}
+
+	return false;
+}
+
+DSO_PUBLIC
+int acvp_load_extension_directory(const char *dir)
+{
+	struct dirent *dentry;
+	DIR *extension_dir = NULL;
+	char filename[FILENAME_MAX];
+	int ret = 0;
+
+	CKNULL_LOG(dir, -EINVAL, "Configuration directory missing\n");
+
+	extension_dir = opendir(dir);
+	CKNULL_LOG(extension_dir, -errno, "Failed to open directory %s\n",
+		   extension_dir);
+
+	while ((dentry = readdir(extension_dir)) != NULL) {
+		if (!acvp_usable_dirent(dentry, "so") &&
+		    !acvp_usable_dirent(dentry, "dylib"))
+			continue;
+
+		/*
+		 * Require the file version to be at least 1.4.0 - the first
+		 * version that has extension support.
+		 *
+		 * We only enforce versioning when loading an entire directory
+		 * since loading individual shared object files are used
+		 * when creating out-of-tree extensions.
+		 */
+		if (!acvp_so_version_ge(dentry, 1, 4, 0)) {
+			logger(LOGGER_ERR, LOGGER_C_ANY,
+			       "Extension %s too old\n", dentry->d_name);
+			ret = -EINVAL;
+			goto out;
+		}
+
+		snprintf(filename, sizeof(filename), "%s/%s", dir,
+			 dentry->d_name);
+		CKINT(acvp_load_extension(filename));
+	}
+
+out:
+	if (extension_dir)
+		closedir(extension_dir);
+	return ret;
+}
