@@ -145,7 +145,9 @@ typedef uint64_t cipher_t;
 #define ACVP_ED25519		(ACVP_CIPHERTYPE_ECC | 0x0000000000001000ULL)
 #define ACVP_ED448		(ACVP_CIPHERTYPE_ECC | 0x0000000000002000ULL)
 #define ACVP_NISTP192		(ACVP_CIPHERTYPE_ECC | 0x0000000000004000ULL)
-#define ACVP_CURVEMASK		(ACVP_CIPHERTYPE_ECC | 0x000000000000ffffULL)
+#define ACVP_NISTK163		(ACVP_CIPHERTYPE_ECC | 0x0000000000008000ULL)
+#define ACVP_NISTB163		(ACVP_CIPHERTYPE_ECC | 0x0000000000010000ULL)
+#define ACVP_CURVEMASK		(ACVP_CIPHERTYPE_ECC | 0x00000000000fffffULL)
 
 #define ACVP_ECDH 		0x0000000001000000ULL
 #define ACVP_DH2048224 		0x0000000002000000ULL
@@ -252,10 +254,12 @@ static const struct cipher_def_map cipher_def_map[] = {
 	{ ACVP_NISTP256, "P-256" },
 	{ ACVP_NISTP384, "P-384" },
 	{ ACVP_NISTP521, "P-521" },
+	{ ACVP_NISTB163, "B-163" },
 	{ ACVP_NISTB233, "B-233" },
 	{ ACVP_NISTB283, "B-283" },
 	{ ACVP_NISTB409, "B-409" },
 	{ ACVP_NISTB571, "B-571" },
+	{ ACVP_NISTK163, "K-163" },
 	{ ACVP_NISTK233, "K-233" },
 	{ ACVP_NISTK283, "K-283" },
 	{ ACVP_NISTK409, "K-409" },
